@@ -3,12 +3,10 @@ SUBROUTINE openfil_bse()
 ! ... This routine opens all files needed to the self consistent run,
 ! ... sets various file names, units, record lengths
   USE wvfct,          ONLY : nbnd, npwx
-  USE io_files,       ONLY : prefix, iunwfc, nwordwfc,nwordatwfc, diropn
+  USE io_files,       ONLY : prefix, iunwfc, nwordwfc, diropn
   USE noncollin_module, ONLY : npol
-  USE basis,            ONLY : natomwfc
   USE ions_base,        ONLY : nat, ityp
   USE noncollin_module, ONLY : noncolin
-  USE upf_ions,         ONLY : n_atom_wfc
 
 
 
@@ -27,8 +25,6 @@ SUBROUTINE openfil_bse()
   IF ( .NOT. exst ) THEN
      call errore ('openfil_pw4gww','file '//TRIM( prefix )//'.wfc'//' not found',1)     
   END IF
-  natomwfc = n_atom_wfc( nat, ityp, noncolin )
-  nwordatwfc = 2*npwx*natomwfc*npol
 
   RETURN
   !
