@@ -35,7 +35,7 @@ subroutine dv_of_drho (dvscf, drhoc)
   USE Coul_cut_2D,       ONLY : do_cutoff_2D  
   USE Coul_cut_2D_ph,    ONLY : cutoff_dv_of_drho 
   USE qpoint,            ONLY : xq
-  USE control_lr,        ONLY : lrpa
+  USE control_lr,        ONLY : lrpa, lnolr
 
   IMPLICIT NONE
   COMPLEX(DP), INTENT(INOUT) :: dvscf(dfftp%nnr, nspin_mag)
@@ -51,10 +51,11 @@ subroutine dv_of_drho (dvscf, drhoc)
   ! counter on r vectors
   ! counter on spin polarizations
   ! counter on g vectors
-  REAL(DP) :: qg2, eh_corr
+  REAL(DP) :: qg2, eh_corr, g2
   ! qg2: the modulus of (q+G)^2
   ! eh_corr: the correction to response Hartree energy due 
   ! to Martyna-Tuckerman correction (calculated, but not used).
+  ! g2: modulus of G^2
   COMPLEX(DP), ALLOCATABLE :: dvaux(:,:), dvhart(:,:), & 
                               dvaux_mt(:), rgtot(:)
   ! dvaux: response XC potential 
@@ -90,32 +91,32 @@ subroutine dv_of_drho (dvscf, drhoc)
   CALL fwfft ('Rho', dvscf(:,1), dfftp)
   !
   IF (do_comp_mt) THEN
-      !
-      ! Response Hartree potential with the Martyna-Tuckerman correction
-      !
-      allocate(dvhart(dfftp%nnr,nspin_mag))
-      dvhart(:,:) = (0.d0,0.d0)
-      !
-      do is = 1, nspin_lsda
+     !
+     ! Response Hartree potential with the Martyna-Tuckerman correction
+     !
+     allocate(dvhart(dfftp%nnr,nspin_mag))
+     dvhart(:,:) = (0.d0,0.d0)
+     !
+     do is = 1, nspin_lsda
         do ig = gstart, ngm
-          qg2 = (g(1,ig)+xq(1))**2 + (g(2,ig)+xq(2))**2 + (g(3,ig)+xq(3))**2
-          dvhart(dfftp%nl(ig),is) = e2 * fpi * dvscf(dfftp%nl(ig),1) / (tpiba2 * qg2)
+           qg2 = (g(1,ig)+xq(1))**2 + (g(2,ig)+xq(2))**2 + (g(3,ig)+xq(3))**2
+           dvhart(dfftp%nl(ig),is) = e2 * fpi * dvscf(dfftp%nl(ig),1) / (tpiba2 * qg2)
         enddo
-      enddo 
-      !
-      ! Add Martyna-Tuckerman correction to response Hartree potential
-      !
-      allocate( dvaux_mt( ngm ), rgtot(ngm) )
-      !
-      ! Total response density
-      !
-      do ig = 1, ngm
-         rgtot(ig) = dvscf(dfftp%nl(ig),1)
-      enddo
-      !
-      CALL wg_corr_h (omega, ngm, rgtot, dvaux_mt, eh_corr)
-      !
-      do is = 1, nspin_lsda
+     enddo 
+     !
+     ! Add Martyna-Tuckerman correction to response Hartree potential
+     !
+     allocate( dvaux_mt( ngm ), rgtot(ngm) )
+     !
+     ! Total response density
+     !
+     do ig = 1, ngm
+        rgtot(ig) = dvscf(dfftp%nl(ig),1)
+     enddo
+     !
+     CALL wg_corr_h (omega, ngm, rgtot, dvaux_mt, eh_corr)
+     !
+     do is = 1, nspin_lsda
         !
         do ig = 1, ngm
            dvhart(dfftp%nl(ig),is)  = dvhart(dfftp%nl(ig),is)  + dvaux_mt(ig)
@@ -130,77 +131,87 @@ subroutine dv_of_drho (dvscf, drhoc)
         !
         CALL invfft ('Rho', dvhart (:,is), dfftp)
         !
-      enddo
-      !
-      ! At the end the two contributions (XC+Hartree) are added
-      ! 
-      dvscf = dvaux + dvhart
-      !
-      deallocate( dvaux_mt, rgtot ) 
-      deallocate(dvhart)
-      !
+     enddo
+     !
+     ! At the end the two contributions (XC+Hartree) are added
+     ! 
+     dvscf = dvaux + dvhart
+     !
+     deallocate( dvaux_mt, rgtot ) 
+     deallocate(dvhart)
+     !
   ELSE
-   !
-   ! Response Hartree potential (without Martyna-Tuckerman correction)
-   !
-   If (gamma_only) then
-      !
-      ! Gamma_only case
-      !
-      allocate(dvhart(dfftp%nnr,nspin_mag))
-      dvhart(:,:) = (0.d0,0.d0)
-      !
-      do is = 1, nspin_lsda
-        do ig = 1, ngm
-           qg2 = (g(1,ig)+xq(1))**2 + (g(2,ig)+xq(2))**2 + (g(3,ig)+xq(3))**2
-           if (qg2 > 1.d-8) then
-              dvhart(dfftp%nl(ig),is) = e2 * fpi * dvscf(dfftp%nl(ig),1) / (tpiba2 * qg2)
-              dvhart(dfftp%nlm(ig),is)=conjg(dvhart(dfftp%nl(ig),is))
-           endif
+     !
+     ! Response Hartree potential (without Martyna-Tuckerman correction)
+     !
+     If (gamma_only) then
+        !
+        ! Gamma_only case
+        !
+        allocate(dvhart(dfftp%nnr,nspin_mag))
+        dvhart(:,:) = (0.d0,0.d0)
+        !
+        do is = 1, nspin_lsda
+           do ig = 1, ngm
+              qg2 = (g(1,ig)+xq(1))**2 + (g(2,ig)+xq(2))**2 + (g(3,ig)+xq(3))**2
+              if (qg2 > 1.d-8) then
+                 dvhart(dfftp%nl(ig),is) = e2 * fpi * dvscf(dfftp%nl(ig),1) / (tpiba2 * qg2)
+                 dvhart(dfftp%nlm(ig),is)=conjg(dvhart(dfftp%nl(ig),is))
+              endif
+           enddo
+           !
+           ! Transformed back to real space
+           !
+           CALL invfft ('Rho', dvhart (:, is), dfftp)
+           !
         enddo
         !
-        ! Transformed back to real space
+        ! At the end the two contributes are added
         !
-        CALL invfft ('Rho', dvhart (:, is), dfftp)
+        dvscf = dvaux + dvhart
         !
-      enddo
-      !
-      ! At the end the two contributes are added
-      !
-      dvscf = dvaux + dvhart
-      !
-      deallocate(dvhart)
-      !
-   else
-      !
-      ! General k points implementation
-      !
-      do is = 1, nspin_lsda
-         CALL fwfft ('Rho', dvaux (:, is), dfftp)
-         IF (do_cutoff_2D) THEN 
-            call cutoff_dv_of_drho(dvaux, is, dvscf)
-         ELSE
-            do ig = 1, ngm
-               qg2 = (g(1,ig)+xq(1))**2 + (g(2,ig)+xq(2))**2 + (g(3,ig)+xq(3))**2
-               if (qg2 > 1.d-8) then
-                  dvaux(dfftp%nl(ig),is) = dvaux(dfftp%nl(ig),is) + &
-                                 & e2 * fpi * dvscf(dfftp%nl(ig),1) / (tpiba2 * qg2)
-               endif
-            enddo
-         ENDIF
-         !
-         ! Transformed back to real space
-         !
-         CALL invfft ('Rho', dvaux (:, is), dfftp)
-         !
-      enddo
-      !
-      ! At the end the two contributes are added
-      !
-      dvscf (:,:) = dvaux (:,:)
-      !
-   endif
-   !
+        deallocate(dvhart)
+        !
+     else
+        !
+        ! General k points implementation
+        !
+        do is = 1, nspin_lsda
+           CALL fwfft ('Rho', dvaux (:, is), dfftp)
+           IF (do_cutoff_2D) THEN 
+              call cutoff_dv_of_drho(dvaux, is, dvscf)
+           ELSE
+              do ig = 1, ngm
+                 qg2 = (g(1,ig)+xq(1))**2 + (g(2,ig)+xq(2))**2 + (g(3,ig)+xq(3))**2
+                 g2  = g(1,ig)**2 + g(2,ig)**2 + g(3,ig)**2
+                 IF (lnolr) THEN
+                    !
+                    IF (g2 > 1d-8 .AND. qg2 > 1.d-8) THEN
+                       dvaux(dfftp%nl(ig),is) = dvaux(dfftp%nl(ig),is) + &
+                                      & e2 * fpi * dvscf(dfftp%nl(ig),1) / (tpiba2 * qg2)
+                    ENDIF
+                    !
+                 ELSE
+                    if (qg2 > 1.d-8) then
+                       dvaux(dfftp%nl(ig),is) = dvaux(dfftp%nl(ig),is) + &
+                                      & e2 * fpi * dvscf(dfftp%nl(ig),1) / (tpiba2 * qg2)
+                    endif
+                 ENDIF
+              enddo
+           ENDIF
+           !
+           ! Transformed back to real space
+           !
+           CALL invfft ('Rho', dvaux (:, is), dfftp)
+           !
+        enddo
+        !
+        ! At the end the two contributes are added
+        !
+        dvscf (:,:) = dvaux (:,:)
+        !
+     endif
+     !
   ENDIF
   !
   deallocate (dvaux)
