@@ -51,7 +51,7 @@ subroutine incdrhoscf (drhoscf, weight, ik, dbecsum, dpsi)
   COMPLEX(DP), ALLOCATABLE :: tg_psi(:), tg_dpsi(:), tg_drho(:)
 
   INTEGER :: npw, npwq, ikk, ikq, itmp
-  INTEGER :: ibnd, ir, ir3, ig, incr, v_siz, idx, ioff, ioff_tg, nxyp
+  INTEGER :: ibnd, ir, ir3, ig, incr, v_siz, idx, ioff, ioff_tg, nxyp, sum_siz
   INTEGER :: right_inc, ntgrp
   ! counters
 
@@ -60,7 +60,6 @@ subroutine incdrhoscf (drhoscf, weight, ik, dbecsum, dpsi)
   INTEGER, POINTER, DEVICE :: nl_d(:)
   !
   nl_d  => dffts%nl_d
-  !$acc update device(evc) 
 #else
   INTEGER, ALLOCATABLE :: nl_d(:)
   !
@@ -93,12 +92,13 @@ subroutine incdrhoscf (drhoscf, weight, ik, dbecsum, dpsi)
      !
   ELSE
      v_siz = dffts%nnr
+     sum_siz = nhm*(nhm+1)/2
   ENDIF
   !
   ! dpsi contains the   perturbed wavefunctions of this k point
   ! evc  contains the unperturbed wavefunctions of this k point
   !
-  !$acc data copyin(dpsi(1:npwx,1:nbnd)) copy(drhoscf(1:v_siz)) create(psi(1:v_siz),dpsic(1:v_siz)) present(igk_k) deviceptr(nl_d) 
+  !$acc data present_or_copyin(dpsi(1:npwx,1:nbnd)) present_or_copy(drhoscf(1:v_siz)) create(psi(1:v_siz),dpsic(1:v_siz)) present(igk_k) deviceptr(nl_d)
   do ibnd = 1, nbnd_occ(ikk), incr
      !
      IF ( dffts%has_task_groups ) THEN
@@ -183,11 +183,11 @@ subroutine incdrhoscf (drhoscf, weight, ik, dbecsum, dpsi)
      ENDIF
      !
   enddo ! loop on bands
-  !$acc end data
   !
   ! Ultrasoft contribution
   ! Calculate dbecsum = <evc|vkb><vkb|dpsi>
   ! 
+  !$acc end data
   CALL addusdbec (ik, weight, dpsi, dbecsum)
   !
   DEALLOCATE(psi)
