@@ -210,10 +210,9 @@ MODULE control_flags
   REAL(DP), PUBLIC  :: &
     ethr               ! the convergence threshold for eigenvalues
   INTEGER, PUBLIC :: &
-    isolve,           &! index selecting Davidson,  CG, PPCG, ParO or RMM diagonalization
+    isolve,           &! index selecting Davidson,  CG, ParO or RMM diagonalization
     david,            &! max dimension of subspace in Davidson diagonalization
     max_cg_iter,      &! maximum number of iterations in a CG call
-    max_ppcg_iter,    &! maximum number of iterations in a PPCG call
     rmm_ndim,         &! max dimension of subspace in RMM-DIIS diagonalization
     gs_nblock          ! blocking size in Gram-Schmidt orthogonalization
   LOGICAL, PUBLIC :: &
