@@ -74,8 +74,6 @@ SUBROUTINE c_bands( iter )
      WRITE( stdout, '(5X,"Davidson diagonalization with overlap")' )
   ELSEIF ( isolve == 1 ) THEN
      WRITE( stdout, '(5X,"CG style diagonalization")')
-  ELSEIF ( isolve == 2 ) THEN
-     WRITE( stdout, '(5X,"PPCG style diagonalization")')
   ELSEIF ( isolve == 3 ) THEN
      WRITE( stdout, '(5X,"ParO style diagonalization")')
   ELSEIF ( isolve == 4 ) THEN
@@ -119,8 +117,10 @@ SUBROUTINE c_bands( iter )
      !
      ! ... Needed for DFT+Hubbard
      !
-     IF ( nks > 1 .AND. lda_plus_u .AND. (Hubbard_projectors .NE. 'pseudo') ) &
-          CALL get_buffer ( wfcU, nwordwfcU, iunhub, ik )
+     IF ( nks > 1 .AND. lda_plus_u .AND. (Hubbard_projectors.NE.'pseudo') ) THEN
+        CALL get_buffer ( wfcU, nwordwfcU, iunhub, ik )
+        !$acc update device(wfcU)
+     END IF
      !
      ! ... diagonalization of bands for k-point ik
      ! ... (skip only in charge self-consistent DFT+DMFT calculations)
@@ -195,14 +195,13 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   USE uspp,                 ONLY : vkb, nkb, okvan
   USE gvect,                ONLY : gstart
   USE wvfct,                ONLY : g2kin, nbndx, et, nbnd, npwx, btype
-  USE control_flags,        ONLY : ethr, lscf, max_cg_iter, max_ppcg_iter, isolve, &
+  USE control_flags,        ONLY : ethr, lscf, max_cg_iter, isolve, &
                                    rmm_ndim, rmm_conv, gs_nblock, &
                                    gamma_only, use_para_diag, use_gpu
+  USE ldaU,                 ONLY : hub_pot_fix
   USE noncollin_module,     ONLY : npol
   USE wavefunctions,        ONLY : evc
   USE g_psi_mod,            ONLY : h_diag, s_diag
-  USE g_psi_mod_gpum,       ONLY : h_diag_d, s_diag_d, using_h_diag, using_s_diag, using_h_diag_d, using_s_diag_d
-  USE scf,                  ONLY : v_of_0
   USE bp,                   ONLY : lelfield, evcel, evcelp, evcelm, bec_evcel, &
                                    gdir, l3dstring, efield, efield_cry
   USE becmod,               ONLY : bec_type, becp, calbec, &
@@ -222,13 +221,11 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   USE plugin_flags,     ONLY : use_oscdft
   USE oscdft_base,      ONLY : oscdft_ctx
   USE oscdft_functions, ONLY : oscdft_h_diag
-  USE oscdft_functions_gpu, ONLY : oscdft_h_diag_gpu
 #endif
   !
   IMPLICIT NONE
   !
   ! please do not capitalize (FORD rules)
-  include 'ks_solver_interfaces.fh'
   !  
   INTEGER, INTENT(IN) :: iter
   !! iteration index
@@ -239,10 +236,9 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   !
   ! ... local variables
   !
-  REAL(KIND=DP) :: cg_iter, ppcg_iter, rmm_iter
-  ! (weighted) number of iterations in Conjugate-Gradient
+  REAL(KIND=DP) :: cg_iter, rmm_iter
   ! (weighted) number of iterations in RMM-DIIS
-  INTEGER :: npw, ig, dav_iter, ntry, notconv, nhpsi
+  INTEGER :: npw, dav_iter, ntry, notconv, nhpsi
   ! number of iterations in Davidson
   ! number or repeated call to diagonalization in case of non convergence
   ! number of notconverged elements
@@ -251,20 +247,11 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   LOGICAL :: lrot
   ! .TRUE. if the wfc have already be rotated
   !
-  INTEGER, PARAMETER :: sbsize = 5, rrstep = 7
-  ! block dimensions used in PPCG 
-  !
-  COMPLEX (DP), POINTER :: hevc_d(:,:), sevc_d(:,:)
-  ! hamiltonian x wavefunctions, only for RMM-DIIS
-  ! overlap x wavefunctions, only for RMM-DIIS 
-#if defined(__CUDA)
-  attributes(DEVICE) :: hevc_d, sevc_d
-#endif
   COMPLEX (DP), POINTER :: hevc(:,:), sevc(:,:)
   !
   ! Davidson and RMM-DIIS diagonalization uses these external routines on groups of nvec bands
   EXTERNAL h_psi, s_psi, g_psi
-  EXTERNAL h_psi_gpu, s_psi_acc, g_psi_gpu
+  EXTERNAL h_psi_gpu, s_psi_acc
   ! subroutine h_psi(npwx,npw,nvec,psi,hpsi)  computes H*psi
   ! subroutine s_psi(npwx,npw,nvec,psi,spsi)  computes S*psi (if needed)
   ! subroutine g_psi(npwx,npw,nvec,psi,eig)   computes G*psi -> psi
@@ -278,16 +265,13 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   ! subroutine s_1psi(npwx,npw,psi,spsi)        computes S*psi (if needed)
   ! In addition to the above the initial wfc rotation uses h_psi, and s_psi
   !------------------------------------------------------------------------
-  ! PPCG diagonalization uses these external routines on groups of bands
   ! subroutine h_psi(npwx,npw,nvec,psi,hpsi)  computes H*psi
   ! subroutine s_psi(npwx,npw,nvec,psi,spsi)  computes S*psi (if needed)
   !------------------------------------------------------------------------
   ! ParO diagonalization uses these external routines on a single band
   ! subroutine hs_1psi(npwx,npw,psi,hpsi,spsi)  computes H*psi and S*psi
-  ! subroutine g_1psi(npwx,npw,psi,eig)         computes G*psi -> psi
   ! In addition to the above the initial wfc rotation uses h_psi, and s_psi
-  external g_1psi
-  external g_1psi_gpu
+  !
   ALLOCATE( h_diag( npwx, npol ), STAT=ierr )
   IF( ierr /= 0 ) &
      CALL errore( ' diag_bands ', ' cannot allocate h_diag ', ABS(ierr) )
@@ -295,8 +279,8 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   ALLOCATE( s_diag( npwx, npol ), STAT=ierr )
   IF( ierr /= 0 ) &
      CALL errore( ' diag_bands ', ' cannot allocate s_diag ', ABS(ierr) )
+  !$acc enter data create (h_diag, s_diag)
   !
-  call using_h_diag(2); call using_s_diag(2)
   ipw=npwx
   CALL mp_sum(ipw, intra_bgrp_comm)
   IF ( nbndx > ipw ) &
@@ -320,13 +304,24 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   ! ... deallocate work space
   !
   CALL deallocate_bec_type_acc( becp )
+  !$acc exit data delete (h_diag, s_diag)
   DEALLOCATE( s_diag )
   DEALLOCATE( h_diag )
-  call using_h_diag(2); call using_s_diag(2)
   !
   IF ( notconv > MAX( 5, nbnd / 4 ) ) THEN
      !
-     CALL errore( 'c_bands', 'too many bands are not converged', 1 )
+     IF ( hub_pot_fix ) THEN
+        ! If perturbing a Hubbard manifold using Hubbard_alpha, 
+        ! need to diagonalize with very tight convergence threshholds
+        ! even during the first iteration.
+        ! Thus, c_bands should not throw an error even though many
+        ! bands might not achieve this convergence criterion.
+        !
+        WRITE( stdout, '(5X,"c_bands: ",I2, " eigenvalues not converged")' ) notconv
+        WRITE( stdout, '(5X,"WARNING: c_bands: not aborting due to active Hubbard alpha")' ) notconv
+     ELSE
+        CALL errore( 'c_bands', 'too many bands are not converged', 1 )
+     ENDIF
      !
   ELSEIF ( notconv > 0 ) THEN
      !
@@ -348,7 +343,7 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
     !
     IMPLICIT NONE
     !
-    INTEGER :: j
+    INTEGER :: ig
     !
     IF ( isolve == 1 .OR. isolve == 2 .OR. isolve == 3 .OR. rmm_use_paro(iter))   THEN
        !
@@ -356,18 +351,13 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        !
        ! ... h_diag is the precondition matrix
        !
-       CALL using_h_diag(2)
        IF ( isolve == 1 .OR. isolve == 2 ) THEN
-          FORALL( ig = 1 : npw )
+          !$acc parallel loop present(g2kin)
+          DO ig = 1, npw 
              h_diag(ig,1) = 1.D0 + g2kin(ig) + SQRT( 1.D0 + ( g2kin(ig) - 1.D0 )**2 )
-          END FORALL
+          END DO
        ELSE
-          FORALL( ig = 1 : npw )
-             h_diag(ig, 1) = g2kin(ig) + v_of_0
-          END FORALL
-          !
-          !$acc update self(vkb)
-          CALL usnldiag( npw, h_diag, s_diag )
+          CALL usnldiag( npw, npol, h_diag, s_diag )
        END IF
        !
        ntry = 0
@@ -382,9 +372,7 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
                 IF (.not. use_gpu) THEN
                    CALL rotate_wfc( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
                 ELSE
-                   !$acc host_data use_device(et)
                    CALL rotate_wfc_gpu( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
-                   !$acc end host_data
                 END IF
                 !
                 avg_iter = avg_iter + 1.D0
@@ -394,13 +382,11 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           !
           IF ( isolve == 1 ) THEN
              IF (.not. use_gpu) THEN
-                CALL using_h_diag(0) ! precondition has intent(in)
                 CALL rcgdiagg( hs_1psi, s_1psi, h_diag, &
                          npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
              ELSE
-                CALL using_h_diag_d(0) ! precondition has intent(in)
-                CALL rcgdiagg_gpu( hs_1psi_gpu, s_1psi_gpu, h_diag_d, &
+                CALL rcgdiagg( hs_1psi_gpu, s_1psi_gpu, h_diag, &
                          npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
                 !
@@ -408,40 +394,18 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
              !
              avg_iter = avg_iter + cg_iter
              !
-          ELSE IF ( isolve == 2 ) THEN
-             IF (.not. use_gpu) THEN
-               CALL using_h_diag(0) ! precondition has intent(in)
-               CALL ppcg_gamma( h_psi, s_psi, okvan, h_diag, &
-                           npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
-                           0.1d0*ethr, max_ppcg_iter, notconv, ppcg_iter, sbsize , rrstep, iter )
-               !
-               avg_iter = avg_iter + ppcg_iter
-               !
-             ELSE
-               CALL using_h_diag_d(0) ! precondition has intent(in)
-               !$acc host_data use_device(evc, et)
-               CALL ppcg_gamma_gpu( h_psi_gpu, s_psi_acc, okvan, h_diag_d, &
-                           npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
-                           0.1d0*ethr, max_ppcg_iter, notconv, ppcg_iter, sbsize , rrstep, iter )
-               !$acc end host_data
-               !
-               avg_iter = avg_iter + ppcg_iter
-               !
-             END IF 
           ELSE
              !
              IF (.not. use_gpu ) THEN
-               CALL using_h_diag(0) ! precondition has intent(in)
-               CALL paro_gamma_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
+               CALL paro_gamma_new( h_psi, s_psi, hs_psi, g_psi, okvan, &
                           npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                !
                avg_iter = avg_iter + nhpsi/float(nbnd) 
                ! write (6,*) ntry, avg_iter, nhpsi
                !
              ELSE
-               CALL using_h_diag_d(0) ! precondition has intent(in)
                !$acc host_data use_device(et)
-               CALL paro_gamma_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_1psi_gpu, okvan, &
+               CALL paro_gamma_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_psi, okvan, &
                           npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                !$acc end host_data
                !
@@ -476,8 +440,6 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        !
        ntry = 0
        !
-       CALL using_h_diag(2);
-       !
        RMM_loop : DO
           !
           lrot = ( iter == 1 .AND. ntry == 0 )
@@ -486,24 +448,21 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
 !          IF ( .NOT. lrot ) THEN
           IF (lrot .AND. .NOT. lscf ) THEN
               !!
-              CALL using_h_diag(2);
-!              CALL using_h_diag(0)
-              FORALL( ig = 1 : npw )
+              !$acc parallel loop present(g2kin)
+              DO ig = 1, npw
                  h_diag(ig,1) = 1.D0 + g2kin(ig) + SQRT( 1.D0 + ( g2kin(ig) - 1.D0 )**2 )
-              END FORALL
+              END DO
               !
               IF (.not. use_gpu ) THEN
-                CALL using_h_diag(0) ! precondition has intent(in)
-                CALL paro_gamma_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
+                CALL paro_gamma_new( h_psi, s_psi, hs_psi, g_psi, okvan, &
                            npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                 !
                 avg_iter = avg_iter + nhpsi/float(nbnd) 
                 ! write (6,*) ntry, avg_iter, nhpsi
                 !
               ELSE
-                CALL using_h_diag_d(0) ! precondition has intent(in)
                 !$acc host_data use_device(et)
-                CALL paro_gamma_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_1psi_gpu, okvan, &
+                CALL paro_gamma_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_psi, okvan, &
                            npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                 !$acc end host_data
                 !$acc update self(et)
@@ -516,15 +475,12 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           ELSE IF ( .NOT. lrot ) THEN
              !
              IF (.not. use_gpu) THEN
-                CALL rotate_xpsi( h_psi, s_psi, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
-                               evc, hevc, sevc, et(:,ik), USE_PARA_DIAG = use_para_diag, GAMMA_ONLY = .TRUE. )
+                CALL rotate_xpsi_driver( h_psi, s_psi, h_psi, s_psi, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
+                               evc, hevc, sevc, et(:,ik), use_para_diag, .TRUE. )
 #if defined(__CUDA)
              ELSE
-                !$acc host_data use_device(et)
-                CALL rotate_xpsi( h_psi, s_psi, h_psi_gpu, s_psi_acc, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
-                               evc, hevc, sevc, et(:,ik), USE_PARA_DIAG = use_para_diag, GAMMA_ONLY = .TRUE.)
-                !$acc end host_data
-                !$acc update self(et)
+                CALL rotate_xpsi_driver( h_psi, s_psi, h_psi_gpu, s_psi_acc, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
+                               evc, hevc, sevc, et(:,ik), use_para_diag, .TRUE.)
 #endif
              END IF
              !
@@ -534,18 +490,13 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           !
           !
           IF (.not. use_gpu) THEN
-            CALL using_h_diag(0) !precondition has intent(in)
             CALL rrmmdiagg( h_psi, s_psi, npwx, npw, nbnd, evc, hevc, sevc, &
                          et(1,ik), g2kin(1), btype(1,ik), ethr, rmm_ndim, &
                          okvan, lrot, exx_is_active(), notconv, rmm_iter )
           ELSE
-             !$acc data present(g2kin)
-             !$acc host_data use_device(g2kin)
-             CALL rrmmdiagg_gpu( h_psi_gpu, s_psi_acc, npwx, npw, nbnd, evc, hevc, sevc, &
+             CALL rrmmdiagg( h_psi_gpu, s_psi_acc, npwx, npw, nbnd, evc, hevc, sevc, &
                           et(1,ik), g2kin, btype(1,ik), ethr, rmm_ndim, &
                           okvan, lrot, exx_is_active(), notconv, rmm_iter )
-             !$acc end host_data 
-             !$acc end data
           END IF
           !
           !
@@ -563,16 +514,8 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        !
        ! ... Gram-Schmidt orthogonalization
        !
-       IF (.not. use_gpu) THEN
-        CALL gram_schmidt_gamma( npwx, npw, nbnd, evc, hevc, sevc, et(1,ik), &
-                        okvan, .TRUE., .TRUE., gs_nblock )
-       ELSE
-          !$acc host_data use_device(evc, hevc, sevc)
-          CALL gram_schmidt_gamma_gpu( npwx, npw, nbnd, evc, hevc, sevc, et(1,ik), &
-                          okvan, .TRUE., .TRUE., gs_nblock )
-          !$acc end host_data
-          !$acc update device(et)
-       END IF
+       CALL gram_schmidt_gamma( npwx, npw, nbnd, evc, hevc, sevc, et(1,ik), &
+                       okvan, .TRUE., .TRUE., gs_nblock )
        !
        avg_iter = avg_iter + 0.5D0
        !
@@ -593,30 +536,10 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        ! ... hamiltonian used in g_psi to evaluate the correction
        ! ... to the trial eigenvectors
        !
-       IF ( .not. use_gpu ) THEN
-          call using_h_diag(2); call using_s_diag(2);
-          !
-          DO j=1, npw
-             h_diag(j, 1) = g2kin(j) + v_of_0
-          END DO
-          !
+       CALL usnldiag( npw, npol, h_diag, s_diag )
 #if defined (__OSCDFT)
-          IF (use_oscdft) CALL oscdft_h_diag(oscdft_ctx)
+       IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==1)) CALL oscdft_h_diag(oscdft_ctx, h_diag)
 #endif
-          CALL usnldiag( npw, h_diag, s_diag )
-       ELSE
-          call using_h_diag_d(2); call using_s_diag_d(2);
-          !
-          !$acc parallel loop present(g2kin, h_diag_d) 
-          DO j=1, npw
-             h_diag_d(j, 1) = g2kin(j) + v_of_0
-          END DO
-          !
-#if defined (__OSCDFT)
-          IF (use_oscdft) CALL oscdft_h_diag_gpu(oscdft_ctx)
-#endif
-          CALL usnldiag_gpu( npw, h_diag_d, s_diag_d )
-       END IF
        !
        ntry = 0
        !
@@ -629,29 +552,29 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
 !                ! make sure that all processors have the same wfc
                 CALL pregterg( h_psi, s_psi, okvan, g_psi, &
                             npw, npwx, nbnd, nbndx, evc, ethr, &
-                            et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi ) !    BEWARE gstart has been removed from call
+                            et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
              ELSE
                 CALL regterg (  h_psi, s_psi, okvan, g_psi, &
                          npw, npwx, nbnd, nbndx, evc, ethr, &
-                         et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi ) !    BEWARE gstart has been removed from call
+                         et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
              END IF
              ! 
           ELSE
-             !$acc host_data use_device(et)
              IF ( use_para_diag ) THEN
-                !$acc host_data use_device(evc)
-                CALL pregterg_gpu( h_psi_gpu, s_psi_acc, okvan, g_psi_gpu, &
+                !$acc host_data use_device(et)
+                CALL pregterg_gpu( h_psi_gpu, s_psi_acc, okvan, g_psi, &
                             npw, npwx, nbnd, nbndx, evc, ethr, &
-                            et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi ) !    BEWARE gstart has been removed from call 
+                            et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
                 !$acc end host_data
                 !
              ELSE
                 !
-                CALL regterg (  h_psi_gpu, s_psi_acc, okvan, g_psi_gpu, &
+                !$acc host_data use_device(et)
+                CALL regterg (  h_psi_gpu, s_psi_acc, okvan, g_psi, &
                          npw, npwx, nbnd, nbndx, evc, ethr, &
-                         et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi ) !    BEWARE gstart has been removed from call
+                         et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
+                !$acc end host_data
              END IF
-             !$acc end host_data
              !$acc update self(et)
           END IF
           !
@@ -683,7 +606,7 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
     INTEGER :: ipol
     REAL(DP) :: eps=0.000001d0
     ! --- Define a small number ---
-    INTEGER :: j
+    INTEGER :: ig
     !
     !write (*,*) ' enter diag_bands_k'; FLUSH(6)
     IF ( lelfield ) THEN
@@ -719,7 +642,7 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
     !
     IF (scissor) evcc = evc
     !
-    !write (*,*) ' current isolve value ( 0 Davidson, 1 CG, 2 PPCG, 3 PARO, 4 RMM)', isolve; FLUSH(6)
+    !write (*,*) ' current isolve value ( 0 Davidson, 1 CG, 3 PARO, 4 RMM)', isolve; FLUSH(6)
     IF ( isolve == 1 .OR. isolve == 2 .OR. isolve == 3 .or. rmm_use_paro(iter)) THEN
        !
        ! ... (Projected Preconditioned) Conjugate-Gradient diagonalization
@@ -728,19 +651,13 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        !
        !write (*,*) ' inside CG solver branch '
        !
-       CALL using_h_diag(2);
-       h_diag = 1.D0
        IF ( isolve == 1 .OR. isolve == 2) THEN
-          FORALL( ig = 1 : npwx )
+          !$acc parallel loop present(g2kin)
+          DO ig = 1, npwx
              h_diag(ig,:) = 1.D0 + g2kin(ig) + SQRT( 1.D0 + ( g2kin(ig) - 1.D0 )**2 )
-          END FORALL
+          END DO
        ELSE
-          FORALL( ig = 1 : npwx )
-             h_diag(ig, :) = g2kin(ig) + v_of_0
-          END FORALL
-          !
-          !$acc update self(vkb)
-          CALL usnldiag( npw, h_diag, s_diag )
+          CALL usnldiag( npw, npol, h_diag, s_diag )
        ENDIF
        !
        ntry = 0
@@ -755,9 +672,7 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
                 IF ( .not. use_gpu ) THEN
                    CALL rotate_wfc( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
                 ELSE
-                   !$acc host_data use_device(et)
                    CALL rotate_wfc_gpu( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
-                   !$acc end host_data
                 END IF
                 !
                 avg_iter = avg_iter + 1.D0
@@ -766,54 +681,28 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           !
           IF ( isolve == 1) then
              IF ( .not. use_gpu ) THEN
-                CALL using_h_diag(0)
                 CALL ccgdiagg( hs_1psi, s_1psi, h_diag, &
                          npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
              ELSE
-                CALL using_h_diag_d(0)
-                CALL ccgdiagg_gpu( hs_1psi_gpu, s_1psi_gpu, h_diag_d, &
+                CALL ccgdiagg( hs_1psi_gpu, s_1psi_gpu, h_diag, &
                          npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
              END IF
              !
              avg_iter = avg_iter + cg_iter
              !
-          ELSE IF ( isolve == 2) then
-             IF ( .not. use_gpu ) THEN
-               CALL using_h_diag(0)
-               ! BEWARE npol should be added to the arguments
-               CALL ppcg_k( h_psi, s_psi, okvan, h_diag, &
-                           npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
-                           0.1d0*ethr, max_ppcg_iter, notconv, ppcg_iter, sbsize , rrstep, iter )
-               !
-               avg_iter = avg_iter + ppcg_iter
-               !
-             ELSE
-               CALL using_h_diag_d(0)
-               ! BEWARE npol should be added to the arguments
-               !$acc host_data use_device(evc, et)
-               CALL ppcg_k_gpu( h_psi_gpu, s_psi_acc, okvan, h_diag_d, &
-                           npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
-                           0.1d0*ethr, max_ppcg_iter, notconv, ppcg_iter, sbsize , rrstep, iter )
-               !$acc end host_data
-               !
-               avg_iter = avg_iter + ppcg_iter
-               !
-             END IF
           ELSE 
              !
              IF ( .not. use_gpu ) THEN
-               CALL using_h_diag(0)
-               CALL paro_k_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
+               CALL paro_k_new( h_psi, s_psi, hs_psi, g_psi, okvan, &
                         npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                !
                avg_iter = avg_iter + nhpsi/float(nbnd) 
                ! write (6,*) ntry, avg_iter, nhpsi
              ELSE
-               CALL using_h_diag_d(0)
                !$acc host_data use_device(et)
-               CALL paro_k_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_1psi_gpu, okvan, &
+               CALL paro_k_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_psi, okvan, &
                         npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                !$acc end host_data
                !
@@ -854,24 +743,17 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
 !edp
 !          IF ( .NOT. lrot ) THEN
           IF (lrot .AND. .NOT. lscf ) THEN
-              !
-              CALL using_h_diag(2)
-              h_diag = 1.D0
-              FORALL( ig = 1 : npwx )
-                 h_diag(ig,:) = g2kin(ig) + v_of_0
-              END FORALL
-              CALL usnldiag(npw, h_diag, s_diag )
+              CALL usnldiag(npw, npol, h_diag, s_diag )
               !
               IF ( .not. use_gpu ) THEN
-                CALL paro_k_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
+                CALL paro_k_new( h_psi, s_psi, hs_psi, g_psi, okvan, &
                          npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                 !
                 avg_iter = avg_iter + nhpsi/float(nbnd) 
                 ! write (6,*) ntry, avg_iter, nhpsi
               ELSE
-                CALL using_h_diag_d(0)
                 !$acc host_data use_device(et)
-                CALL paro_k_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_1psi_gpu, okvan, &
+                CALL paro_k_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_psi, okvan, &
                          npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                 !$acc end host_data
                 !$acc update self(et)
@@ -884,17 +766,14 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           ELSE IF ( .NOT. lrot ) THEN
              !
              IF ( .not. use_gpu ) THEN
-                CALL rotate_xpsi( h_psi, s_psi, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
+                CALL rotate_xpsi_driver( h_psi, s_psi, h_psi, s_psi, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
                                   evc, hevc, sevc, et(:,ik), & 
-                                  USE_PARA_DIAG = use_para_diag, GAMMA_ONLY = gamma_only )
+                                  use_para_diag, gamma_only )
 #if defined(__CUDA)
              ELSE
-                !$acc host_data use_device(et)
-                CALL rotate_xpsi( h_psi, s_psi, h_psi_gpu, s_psi_acc, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
+                CALL rotate_xpsi_driver( h_psi, s_psi, h_psi_gpu, s_psi_acc, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
                                   evc, hevc, sevc, et(:,ik), &
-                                  USE_PARA_DIAG = use_para_diag, GAMMA_ONLY = gamma_only )
-                !$acc end host_data
-                !$acc update self(et)
+                                  use_para_diag, gamma_only )
 #endif
              END IF
              !
@@ -903,18 +782,13 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           END IF
           !
           IF ( .not. use_gpu ) THEN
-             CALL using_h_diag(0)
              CALL crmmdiagg( h_psi, s_psi, npwx, npw, nbnd, npol, evc, hevc, sevc, &
                              et(1,ik), g2kin(1), btype(1,ik), ethr, rmm_ndim, &
                              okvan, lrot, exx_is_active(), notconv, rmm_iter )
           ELSE
-             !$acc data present(g2kin)
-             !$acc host_data use_device(g2kin)
-             CALL crmmdiagg_gpu( h_psi_gpu, s_psi_acc, npwx, npw, nbnd, npol, evc, hevc, sevc, &
+             CALL crmmdiagg( h_psi_gpu, s_psi_acc, npwx, npw, nbnd, npol, evc, hevc, sevc, &
                              et(1,ik), g2kin(1), btype(1,ik), ethr, rmm_ndim, &
                              okvan, lrot, exx_is_active(), notconv, rmm_iter )
-             !$acc end host_data
-             !$acc end data
           END IF
           !
           IF ( lscf .AND. ( .NOT. rmm_conv ) ) notconv = 0
@@ -931,16 +805,8 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        !
        ! ... Gram-Schmidt orthogonalization
        !
-       IF ( .not. use_gpu ) THEN
-          CALL gram_schmidt_k( npwx, npw, nbnd, npol, evc, hevc, sevc, et(1,ik), &
-                             okvan, .TRUE., .TRUE., gs_nblock )
-       ELSE
-          !$acc host_data use_device(evc, hevc, sevc)
-          CALL gram_schmidt_k_gpu( npwx, npw, nbnd, npol, evc, hevc, sevc, et(1,ik), &
-                             okvan, .TRUE., .TRUE., gs_nblock )
-          !$acc end host_data
-          !$acc update device(et)
-       END IF
+       CALL gram_schmidt_k( npwx, npw, nbnd, npol, evc, hevc, sevc, et(1,ik), &
+                          okvan, .TRUE., .TRUE., gs_nblock )
        !
        avg_iter = avg_iter + 0.5D0
        !
@@ -953,7 +819,6 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           NULLIFY( sevc )
        END IF
        !
-       !
     ELSE
        !
        ! ... Davidson diagonalization
@@ -962,40 +827,10 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        ! ... hamiltonian used in g_psi to evaluate the correction
        ! ... to the trial eigenvectors
        !
-       IF ( .not. use_gpu ) THEN
-          !
-          CALL using_h_diag(2);
-          !
-          DO ipol = 1, npol
-             !
-             h_diag(1:npw, ipol) = g2kin(1:npw) + v_of_0
-             !
-          END DO
+       CALL usnldiag( npw, npol, h_diag, s_diag )
 #if defined (__OSCDFT)
-          IF (use_oscdft) CALL oscdft_h_diag(oscdft_ctx)
+       IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==1)) CALL oscdft_h_diag(oscdft_ctx, h_diag)
 #endif
-          !
-          call using_s_diag(2);
-          CALL usnldiag( npw, h_diag, s_diag )
-       ELSE
-          !
-          CALL using_h_diag_d(2)
-          !
-          DO ipol = 1, npol
-             !
-             !$acc parallel loop present(g2kin, h_diag_d) 
-             DO j = 1, npw
-                h_diag_d(j, ipol) = g2kin(j) + v_of_0
-             END DO
-             !
-          END DO
-          !
-#if defined (__OSCDFT)
-          IF (use_oscdft) CALL oscdft_h_diag_gpu(oscdft_ctx)
-#endif
-          CALL using_s_diag_d(2); CALL using_h_diag_d(1)
-          CALL usnldiag_gpu( npw, h_diag_d, s_diag_d )
-       END IF
        !
        ntry = 0
        !
@@ -1017,22 +852,22 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
                                et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
              END IF
           ELSE
-             !$acc host_data use_device(et)
              IF ( use_para_diag ) then
                 !
-                !$acc host_data use_device(evc)
-                CALL pcegterg_gpu( h_psi_gpu, s_psi_acc, okvan, g_psi_gpu, &
+                !$acc host_data use_device(et)
+                CALL pcegterg_gpu( h_psi_gpu, s_psi_acc, okvan, g_psi, &
                                npw, npwx, nbnd, nbndx, npol, evc, ethr, &
                                et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
                 !$acc end host_data
                 !
              ELSE
                 !
-                CALL cegterg ( h_psi_gpu, s_psi_acc, okvan, g_psi_gpu, &
+                !$acc host_data use_device(et)
+                CALL cegterg ( h_psi_gpu, s_psi_acc, okvan, g_psi, &
                                npw, npwx, nbnd, nbndx, npol, evc, ethr, &
                                et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
+                !$acc end host_data 
              END IF
-             !$acc end host_data 
              !$acc update self(et)
           END IF
           !
@@ -1204,8 +1039,6 @@ SUBROUTINE c_bands_nscf( )
      WRITE( stdout, '(5X,"Davidson diagonalization with overlap")' )
   ELSEIF ( isolve == 1 ) THEN
      WRITE( stdout, '(5X,"CG style diagonalization")' )
-  ELSEIF ( isolve == 2 ) THEN
-     WRITE( stdout, '(5X,"PPCG style diagonalization")' )
   ELSEIF ( isolve == 3 ) THEN
      WRITE( stdout, '(5X,"ParO style diagonalization")')
   ELSEIF ( isolve == 4 ) THEN
@@ -1235,8 +1068,10 @@ SUBROUTINE c_bands_nscf( )
      !
      ! ... Needed for DFT+Hubbard
      !
-     IF ( nks > 1 .AND. lda_plus_u .AND. (Hubbard_projectors .NE. 'pseudo') ) &
-          CALL get_buffer( wfcU, nwordwfcU, iunhub, ik )
+     IF ( nks > 1 .AND. lda_plus_u .AND. (Hubbard_projectors.NE.'pseudo') ) THEN
+        CALL get_buffer ( wfcU, nwordwfcU, iunhub, ik )
+        !$acc update device(wfcU)
+     END IF
      !
      ! ... calculate starting  wavefunctions
      !
