@@ -22,7 +22,7 @@ SUBROUTINE hinit1()
   USE lsda_mod,            ONLY : nspin
   USE noncollin_module,    ONLY : report
   USE scf,                 ONLY : vrs, vltot, v, kedtau
-  USE control_flags,       ONLY : tqr, use_gpu
+  USE control_flags,       ONLY : tqr, use_gpu, io_level
   USE realus,              ONLY : generate_qpointlist, betapointlist, &
                                   init_realspace_vars, real_space
   USE wannier_new,         ONLY : use_wannier
@@ -32,7 +32,6 @@ SUBROUTINE hinit1()
   USE paw_onecenter,       ONLY : paw_potential
   USE paw_symmetry,        ONLY : paw_symmetrize_ddd
   USE dfunct,              ONLY : newd
-  USE dfunct_gpum,         ONLY : newd_gpu
   USE exx_base,            ONLY : coulomb_fac, coulomb_done
   !
   USE ener,                ONLY : esol, vsol
@@ -123,13 +122,18 @@ SUBROUTINE hinit1()
      CALL PAW_symmetrize_ddd( ddd_paw )
   ENDIF
   ! 
-  IF (.not. use_gpu) CALL newd()
-  IF (      use_gpu) CALL newd_gpu()
+  CALL newd()
   !
   ! ... and recalculate the products of the S with the atomic wfcs used 
   ! ... in DFT+Hubbard calculations
   !
-  IF ( lda_plus_u  ) CALL orthoUwfc(.FALSE.)
+  IF ( lda_plus_u  ) THEN
+     IF (io_level>=1) THEN
+        CALL orthoUwfc(.TRUE.)
+     ELSE
+        CALL orthoUwfc(.FALSE.)
+     ENDIF
+  ENDIF
   IF ( use_wannier ) CALL orthoatwfc( .TRUE. )
   !
   ! ... The following line forces recalculation of terms used by EXX
