@@ -50,6 +50,7 @@ CONTAINS
   !! Tetrahedron method according to P. E. Bloechl et al, PRB49, 16223 (1994).
   !
   USE kinds,      ONLY : DP
+  USE noncollin_module,   ONLY : colin_mag
   !
   IMPLICIT NONE
   ! 
@@ -97,6 +98,7 @@ CONTAINS
              n1, n2, n3, n4, n5, n6, n7, n8
   INTEGER, ALLOCATABLE:: equiv(:)
   !
+  !
   ntetra  = 6*nk1*nk2*nk3
   nntetra = 4
   !
@@ -136,7 +138,7 @@ CONTAINS
                        s(i,2,ns) * xk(2,n) + &
                        s(i,3,ns) * xk(3,n)
            ENDDO
-           IF (t_rev(ns) == 1) xkr = -xkr
+           IF (t_rev(ns) == 1 .AND. colin_mag < 2) xkr = -xkr
            !  xkr is the n-th irreducible k-point rotated wrt the ns-th symmetry
            DO i = 1, 3
               deltap(i) = xkr(i)-xkg(i,nk) - NINT(xkr(i)-xkg(i,nk))
@@ -250,6 +252,7 @@ CONTAINS
   !! This rouotine sets the corners and additional points for each tetrahedron.
   !
   USE io_global,    ONLY : stdout
+  USE noncollin_module,   ONLY : colin_mag
   !
   IMPLICIT NONE
   !
@@ -288,6 +291,7 @@ CONTAINS
   !
   REAL(DP), INTENT(INOUT) :: xk(3,npk)
   !! k points [2 pi / a]
+  ! 
   !
   ! ... local variables
   !
@@ -299,7 +303,7 @@ CONTAINS
   !
   REAL(DP) :: xkr(3), l(4), bvec2(3,3), bvec3(3,4), xkg(3,nk1*nk2*nk3), &
               deltap(3), deltam(3)
-  !
+  ! 
   ! Take the shortest diagonal line as the "shaft" of tetrahedral devision
   !
   bvec2(1:3,1) = bg(1:3,1) / REAL(nk1, dp)
@@ -451,7 +455,7 @@ CONTAINS
         DO isym = 1, nsym
            !
            xkr(1:3) = MATMUL(REAL(s(1:3, 1:3, isym), dp), xk(1:3, jk))
-           IF (t_rev(isym) == 1) xkr(1:3) = - xkr(1:3)
+           IF (t_rev(isym) == 1 .AND. colin_mag < 2) xkr(1:3) = - xkr(1:3)
            !  xkr is the n-th irreducible k-point rotated wrt the ns-th symmetry
            deltap(1:3) = xkr(1:3) - xkg(1:3,ik) - NINT(xkr(1:3) - xkg(1:3,ik))
            deltam(1:3) = xkr(1:3) + xkg(1:3,ik) - NINT(xkr(1:3) + xkg(1:3,ik))
@@ -1289,8 +1293,6 @@ SUBROUTINE opt_tetra_partialdos( nspin0, kresolveddos, ne, natomwfc, nkseff, &
   USE mp_images,           ONLY : intra_image_comm
   USE mp,                 ONLY : mp_sum
   !
-  USE wvfct_gpum,         ONLY: using_et
-  !
   IMPLICIT NONE
   !
   LOGICAL :: kresolveddos
@@ -1314,7 +1316,6 @@ SUBROUTINE opt_tetra_partialdos( nspin0, kresolveddos, ne, natomwfc, nkseff, &
      nspin1 = 1
   ENDIF
   !
-  CALL using_et(0)
   ALLOCATE(et_col(nbnd,nkstot), proj_col(natomwfc,nbnd,nkstot))
   CALL poolcollect(nbnd, nks, et, nkstot, et_col)
   CALL poolcollect(nbnd*natomwfc, nks, proj, nkstot, proj_col)
