@@ -64,21 +64,133 @@ MODULE xdm_module
   ! free volumes
   REAL(DP), ALLOCATABLE :: afree(:)
 
-  ! free atomic polarizabilities from CRC handbook, 88th ed. (ang^3->bohr^3)
-  REAL(DP), PARAMETER :: alpha_free(1:102) = (/0.6668_DP, 0.2051_DP, 24.3300_DP, 5.6000_DP,&
-     3.0300_DP, 1.7600_DP, 1.1000_DP, 0.8020_DP, 0.5570_DP, 0.3956_DP, 24.1100_DP, 10.6000_DP,&
-     6.8000_DP, 5.3800_DP, 3.6300_DP, 2.9000_DP, 2.1800_DP, 1.6411_DP, 43.4000_DP, 22.8000_DP,&
-     17.8000_DP, 14.6000_DP, 12.4000_DP, 11.6000_DP, 9.4000_DP, 8.4000_DP, 7.5000_DP, 6.8000_DP,&
-     6.2000_DP, 5.7500_DP, 8.1200_DP, 6.0700_DP, 4.3100_DP, 3.7700_DP, 3.0500_DP, 2.4844_DP,&
-     47.3000_DP, 27.6000_DP, 22.7000_DP, 17.9000_DP, 15.7000_DP, 12.8000_DP, 11.4000_DP, 9.6000_DP,&
-     8.6000_DP, 4.8000_DP, 7.2000_DP, 7.3600_DP, 10.2000_DP, 7.7000_DP, 6.6000_DP, 5.5000_DP,&
-     5.3500_DP, 4.0440_DP, 59.4200_DP, 39.7000_DP, 31.1000_DP, 29.6000_DP, 28.2000_DP, 31.4000_DP,&
-     30.1000_DP, 28.8000_DP, 27.7000_DP, 23.5000_DP, 25.5000_DP, 24.5000_DP, 23.6000_DP, 22.7000_DP,&
-     21.8000_DP, 21.0000_DP, 21.9000_DP, 16.2000_DP, 13.1000_DP, 11.1000_DP, 9.7000_DP, 8.5000_DP,&
-     7.6000_DP, 6.5000_DP, 5.8000_DP, 5.0200_DP, 7.6000_DP, 6.8000_DP, 7.4000_DP, 6.8000_DP, 6.0000_DP,&
-     5.3000_DP, 48.6000_DP, 38.3000_DP, 32.1000_DP, 32.1000_DP, 25.4000_DP, 24.9000_DP, 24.8000_DP,&
-     24.5000_DP, 23.3000_DP, 23.0000_DP, 22.7000_DP, 20.5000_DP, 19.7000_DP, 23.8000_DP, 18.2000_DP,&
-     17.5000_DP /) / bohr_radius_angs**3
+  ! free atomic polarizabilities from CRC handbook, 88th ed. (ang^3->bohr^3),
+  ! up to Z=102;
+  ! for Z=103-120 polarizabilities taken  from:
+  !      https://ctcp.massey.ac.nz/2023Tablepol.pdf  
+  !   of   https://doi.org/10.1080/00268976.2018.1535143 
+  REAL(DP), PARAMETER :: alpha_free(1:120) = (/   &
+     0.6668_DP,   &  ! H(Z=1)
+     0.2051_DP,   &  ! He(Z=2)
+     24.3300_DP,  &  ! Li(Z=3)
+     5.6000_DP,   &  ! Be(Z=4)
+     3.0300_DP,   &  ! B (Z=5)
+     1.7600_DP,   &  ! C (Z=6)
+     1.1000_DP,   &  ! N (Z=7)
+     0.8020_DP,   &  ! O (Z=8)
+     0.5570_DP,   &  ! F (Z=9)
+     0.3956_DP,   &  ! Ne (Z=10)
+     24.1100_DP,  &  ! Na (Z=11)
+     10.6000_DP,  &  ! Mg (Z=12)
+     6.8000_DP,   &  ! Al (Z=13)
+     5.3800_DP,   &  ! Si (Z=14)
+     3.6300_DP,   &  ! P  (Z=15)
+     2.9000_DP,   &  ! S  (Z=16)
+     2.1800_DP,   &  ! Cl (Z=17)
+     1.6411_DP,   &  ! Ar (Z=18)
+     43.4000_DP,  &  ! K  (Z=19)
+     22.8000_DP,  &  ! Ca (Z=20)
+     17.8000_DP,  &  ! Sc (Z=21)
+     14.6000_DP,  &  ! Ti (Z=22)
+     12.4000_DP,  &  ! V  (Z=23)
+     11.6000_DP,  &  ! Cr (Z=24)
+     9.4000_DP,   &  ! Mn (Z=25)
+     8.4000_DP,   &  ! Fe (Z=26)
+     7.5000_DP,   &  ! Co (Z=27)
+     6.8000_DP,   &  ! Ni (Z=28)
+     6.2000_DP,   &  ! Cu (Z=29)
+     5.7500_DP,   &  ! Zn (Z=30)
+     8.1200_DP,   &  ! Ga (Z=31)
+     6.0700_DP,   &  ! Ge (Z=32)
+     4.3100_DP,   &  ! As (Z=33)
+     3.7700_DP,   &  ! Se (Z=34)
+     3.0500_DP,   &  ! Br (Z=35)
+     2.4844_DP,   &  ! Kr (Z=36)
+     47.3000_DP,  &  ! Rb (Z=37)
+     27.6000_DP,  &  ! Sr (Z=38)
+     22.7000_DP,  &  !  Y (Z=39)
+     17.9000_DP,  &  ! Zr (Z=40)
+     15.7000_DP,  &  ! Nb (Z=41)
+     12.8000_DP,  &  ! Mo (Z=42)
+     11.4000_DP,  &  ! Tc (Z=43)
+     9.6000_DP,   &  ! Ru (Z=44)
+     8.6000_DP,   &  ! Rh (Z=45)
+     4.8000_DP,   &  ! Pd (Z=46)
+     7.2000_DP,   &  ! Ag (Z=47)
+     7.3600_DP,   &  ! Cd (Z=48)
+     10.2000_DP,  &  ! In (Z=49)
+     7.7000_DP,   &  ! Sn (Z=50)
+     6.6000_DP,   &  ! Sb (Z=51)
+     5.5000_DP,   &  ! Te (Z=52)
+     5.3500_DP,   &  ! I  (Z=53)
+     4.0440_DP,   &  ! Xe (Z=54)
+     59.4200_DP,  &  ! Cs (Z=55)
+     39.7000_DP,  &  ! Ba (Z=56)
+     31.1000_DP,  &  ! La (Z=57)
+     29.6000_DP,  &  ! Ce (Z=58)
+     28.2000_DP,  &  ! Pr (Z=59)
+     31.4000_DP,  &  ! Nd (Z=60)
+     30.1000_DP,  &  ! Pm (Z=61)
+     28.8000_DP,  &  ! Sm (Z=62)
+     27.7000_DP,  &  ! Eu (Z=63)
+     23.5000_DP,  &  ! Gd (Z=64)
+     25.5000_DP,  &  ! Tb (Z=65)
+     24.5000_DP,  &  ! Dy (Z=66)
+     23.6000_DP,  &  ! Ho (Z=67)
+     22.7000_DP,  &  ! Er (Z=68)
+     21.8000_DP,  &  ! Tm (Z=69)
+     21.0000_DP,  &  ! Yb (Z=70)
+     21.9000_DP,  &  ! Lu (Z=71)
+     16.2000_DP,  &  ! Hf (Z=72)
+     13.1000_DP,  &  ! Ta (Z=73)
+     11.1000_DP,  &  ! W  (Z=74)
+     9.7000_DP,   &  ! Re (Z=75)
+     8.5000_DP,   &  ! Os (Z=76)
+     7.6000_DP,   &  ! Ir (Z=77)
+     6.5000_DP,   &  ! Pt (Z=78)
+     5.8000_DP,   &  ! Au (Z=79)
+     5.0200_DP,   &  ! Hg (Z=80)
+     7.6000_DP,   &  ! Tl (Z=81)
+     6.8000_DP,   &  ! Pb (Z=82)
+     7.4000_DP,   &  ! Bi (Z=83)
+     6.8000_DP,   &  ! Po (Z=84)
+     6.0000_DP,   &  ! At (Z=85)
+     5.3000_DP,   &  ! Rn (Z=86) 
+     48.6000_DP,  &  ! Fr (Z=87)
+     38.3000_DP,  &  ! Ra (Z=88)
+     32.1000_DP,  &  ! Ac (Z=89)
+     32.1000_DP,  &  ! Th (Z=90) 
+     25.4000_DP,  &  ! Pa (Z=91)
+     24.9000_DP,  &  ! U (Z=92)
+     24.8000_DP,  &  ! Np (Z=93)
+     24.5000_DP,  &  ! Pu (Z=94)
+     23.3000_DP,  &  ! Am (Z=95)
+     23.0000_DP,  &  ! Cm (Z=96)
+     22.7000_DP,  &  ! Bk (Z=97)
+     20.5000_DP,  &  ! Cf (Z=98)
+     19.7000_DP,  &  ! Es (Z=99)
+     23.8000_DP,  &  ! Fm (Z=100)
+     18.2000_DP,  &  ! Md (Z=101)
+     17.5000_DP,  &  ! No (Z=102)
+     47.4191_DP,  &  ! Lr (Z=103)
+     16.5967_DP,  &  ! Rf (Z=104)
+      6.2238_DP,  &  ! Db (Z=105)
+      5.9274_DP,  &  ! Sg (Z=106)
+      5.6310_DP,  &  ! Bh (Z=107)
+      5.3346_DP,  &  ! Hs (Z=108)
+      5.0383_DP,  &  ! Mt (Z=109)
+      4.7419_DP,  &  ! Ds (Z=110)
+      4.7419_DP,  &  ! Rg (Z=111)
+      4.1492_DP,  &  ! Cn (Z=112)
+      4.2974_DP,  &  ! Nh (Z=113)
+      4.5937_DP,  &  ! Fl (Z=114)
+     10.5211_DP,  &  ! Mc (Z=115)
+      9.9284_DP,  &  ! Lv (Z=116)
+     11.2620_DP,  &  ! Ts (Z=117)
+      8.5947_DP,  &  ! Og (Z=118)
+     25.0432_DP,  &  ! Unu(Un) (Z=119)
+     23.5614_DP   &  ! Ubn(Ub) (Z=120)
+      /) / bohr_radius_angs**3
 
   ! factorials and lambda_l1,l2
   REAL(DP), PARAMETER :: fact(0:8) = REAL((/1,1,2,6,24,120,720,5040,40320/),DP)
@@ -97,9 +209,13 @@ CONTAINS
     USE atom,        ONLY : rgrid, msh
     USE splinelib,   ONLY : spline
     !
-    INTEGER :: i, j, ialloc, nn
+    INTEGER :: i, j, ialloc, n1, nn
     REAL(DP), ALLOCATABLE :: d1y(:), d2y(:)
     !
+
+    ! No UPF information, nothing to initialize (e.g. when called from pp.x)
+    IF (.NOT.ALLOCATED(upf)) RETURN
+
     CALL start_clock('init_xdm')
 
     ispaw = ALL(upf(1:ntyp)%tpawp)
@@ -125,11 +241,13 @@ CONTAINS
     IF (ialloc /= 0) CALL alloc_failed("rcore")
     DO i = 1, ntyp
        nn = msh(i)
-       IF (ispaw) THEN
-          rfree(1:nn,i) = upf(i)%rho_at(1:nn) / (fpi*rgrid(i)%r(1:nn)**2) + upf(i)%paw%ae_rho_atc(1:nn)
-       ELSE
-          rfree(1:nn,i) = upf(i)%rho_at(1:nn) / (fpi*rgrid(i)%r(1:nn)**2)
+       n1 = 1
+       IF ( rgrid(i)%r(n1) < 1.0D-8 ) THEN
+          rfree(n1,i) = 0.0_dp
+          n1 = n1 + 1
        END IF
+       rfree(n1:nn,i) = upf(i)%rho_at(n1:nn) / (fpi*rgrid(i)%r(n1:nn)**2) 
+       IF (ispaw) rfree(1:nn,i) = rfree(1:nn,i) + upf(i)%paw%ae_rho_atc(1:nn)
        CALL radial_gradient(rfree(1:nn,i),d1y(1:nn),rgrid(i)%r(1:nn),nn,1)
        CALL radial_gradient(d1y(1:nn),d2y(1:nn),rgrid(i)%r(1:nn),nn,1)
        CALL spline(rgrid(i)%r(1:nn),rfree(1:nn,i),d1y(1),d2y(1),w2free(1:nn,i))
@@ -318,7 +436,7 @@ CONTAINS
        DO ispin = 1, nspin
           ! spin-contribution to rhoae; this is used in the calculation of the volume
           IF (.NOT.ispaw) THEN
-             rhoae = rho%of_r(:,ispin)
+             rhoae = MAX(rho%of_r(:,ispin),0._DP)
           END IF
           ALLOCATE(gaux(3,dfftp%nnr),ggaux(3,3,dfftp%nnr),STAT=ialloc)
           IF (ialloc /= 0) CALL alloc_failed("gaux, ggaux")
