@@ -20,7 +20,7 @@ PROGRAM lr_dav_main
   USE lr_variables,          ONLY : restart, restart_step,&
        evc1,n_ipol, d0psi, &
        no_hxc, nbnd_total, &
-       revc0, lr_io_level, code1,davidson
+       lr_io_level, code1,davidson
   USE ions_base,             ONLY : tau,nat,atm,ityp
   USE environment,           ONLY : environment_start
   USE mp_global,             ONLY : nimage, mp_startup, inter_bgrp_comm, &
@@ -32,7 +32,6 @@ PROGRAM lr_dav_main
   USE xc_lib,                ONLY : xclib_dft_is
   use lr_dav_routines
   use lr_dav_variables
-  use lr_dav_debug
   !
 #if defined (__ENVIRON)
   USE plugin_flags,          ONLY : use_environ
@@ -96,11 +95,9 @@ PROGRAM lr_dav_main
   CALL lr_dv_setup()
 
   !   Davidson loop
-  !$acc data copyin(revc0(:,:,:))
   if (precondition) write(stdout,'(/5x,"Precondition is used in the algorithm,")')
   do while (.not. dav_conv .and. dav_iter .lt. max_iter)
     dav_iter=dav_iter+1
-      if(if_check_orth) call check_orth()
       ! In one david step, M_C,M_D and M_CD are first constructed;then will be
         ! solved rigorously; then the solution in the subspace left_sub() will
         ! be transformed into full space left_full()
@@ -116,15 +113,9 @@ PROGRAM lr_dav_main
       endif
       !
   enddo
-  !$acc end data
-  ! call check_hermitian()
   ! Extract physical meaning from the solution
   if ( check_stop_now() ) goto 100
   call interpret_eign('END')
-  ! The check_orth at the end may take quite a lot of time in the case of 
-  ! USPP because we didn't store the S* vector basis. Turn this step on only
-  ! in cases of debugging
-  ! call check_orth() 
   if(lplot_drho) call plot_drho()
 
 100 continue
